@@ -53,8 +53,10 @@ class SocketIO(object):
                     multiple clusters of SocketIO processes need to use the
                     same message queue without interfering with each other, then
                     each cluster should use a different channel.
-    :param resource: The SocketIO resource name. Defaults to ``'socket.io'``.
-                     Leave this as is unless you know what you are doing.
+    :param path: The path where the Socket.IO server is exposed. Defaults to
+                 ``'socket.io'``. Leave this as is unless you know what you are
+                 doing.
+    :param resource: Alias to ``path``.
     :param kwargs: Socket.IO and Engine.IO server options.
 
     The Socket.IO server options are detailed below:
@@ -84,7 +86,7 @@ class SocketIO(object):
                        options are "threading", "eventlet" and "gevent". If
                        this argument is not given, "eventlet" is tried first,
                        then "gevent", and finally "threading". The websocket
-                       transport is not supported in "ithreading" mode.
+                       transport is not supported in "threading" mode.
     :param ping_timeout: The time in seconds that the client waits for the
                          server to respond before disconnecting.
     :param ping_interval: The interval in seconds at which the client pings
@@ -138,10 +140,11 @@ class SocketIO(object):
                                     write_only=write_only)
                 self.server_options['client_manager'] = queue
 
-        resource = kwargs.pop('resource', 'socket.io')
+        resource = kwargs.pop('path', kwargs.pop('resource', 'socket.io'))
         if resource.startswith('/'):
             resource = resource[1:]
         self.server = socketio.Server(**self.server_options)
+        self.async_mode = self.server.async_mode
         for handler in self.handlers:
             self.server.on(handler[0], handler[1], namespace=handler[2])
         if app is not None:
@@ -174,6 +177,9 @@ class SocketIO(object):
 
         def decorator(handler):
             def _handler(sid, *args):
+                if sid not in self.server.environ:
+                    # we don't have record of this client, ignore this event
+                    return '', 400
                 app = self.server.environ[sid]['flask.app']
                 with app.request_context(self.server.environ[sid]):
                     if 'saved_session' in self.server.environ[sid]:
@@ -195,7 +201,7 @@ class SocketIO(object):
                             raise
                         type, value, traceback = sys.exc_info()
                         return err_handler(value)
-                    if flask.session.modified:
+                    if flask.session.modified and sid in self.server.environ:
                         self.server.environ[sid]['saved_session'] = {}
                         self._copy_session(
                             flask.session,
@@ -477,6 +483,33 @@ class SocketIO(object):
         elif self.server.eio.async_mode == 'gevent':
             self.wsgi_server.stop()
 
+    def start_background_task(self, target, *args, **kwargs):
+        """Start a background task using the appropriate async model.
+
+        This is a utility function that applications can use to start a
+        background task using the method that is compatible with the
+        selected async mode.
+
+        :param target: the target function to execute.
+        :param args: arguments to pass to the function.
+        :param kwargs: keyword arguments to pass to the function.
+
+        This function returns an object compatible with the `Thread` class in
+        the Python standard library. The `start()` method on this object is
+        already called by this function.
+        """
+        return self.server.start_background_task(target, *args, **kwargs)
+
+    def sleep(self, seconds=0):
+        """Sleep for the requested amount of time using the appropriate async
+        model.
+
+        This is a utility function that applications can use to put a task to
+        sleep without having to worry about using the correct call for the
+        selected async mode.
+        """
+        return self.server.sleep(seconds)
+
     def test_client(self, app, namespace=None):
         """Return a simple SocketIO client that can be used for unit tests."""
         return SocketIOTestClient(app, self, namespace)
@@ -552,7 +585,10 @@ def send(message, **kwargs):
                          addressing a room, or ``False`` to send to everyone
                          but the sender.
     """
-    namespace = kwargs.get('namespace', flask.request.namespace)
+    if 'namespace' in kwargs:
+        namespace = kwargs['namespace']
+    else:
+        namespace = flask.request.namespace
     callback = kwargs.get('callback')
     broadcast = kwargs.get('broadcast')
     room = kwargs.get('room')
